@@ -15,15 +15,11 @@ namespace phpbb\template\twig\node;
 
 abstract class includeasset extends \Twig_Node
 {
-	/** @var \Twig_Environment */
-	protected $environment;
-
-	public function __construct(\Twig_Node_Expression $expr, \phpbb\template\twig\environment $environment, $lineno, $tag = null)
+	public function __construct(\Twig_Node_Expression $expr, $lineno, $tag = null)
 	{
-		$this->environment = $environment;
-
 		parent::__construct(array('expr' => $expr), array(), $lineno, $tag);
 	}
+
 	/**
 	* Compiles the node to PHP.
 	*
@@ -33,27 +29,30 @@ abstract class includeasset extends \Twig_Node
 	{
 		$compiler->addDebugInfo($this);
 
-		$config = $this->environment->get_phpbb_config();
-
 		$compiler
 			->write("\$asset_file = ")
 			->subcompile($this->getNode('expr'))
 			->raw(";\n")
-			->write("\$asset = new \phpbb\\template\\asset(\$asset_file, \$this->getEnvironment()->get_path_helper(), \$this->getEnvironment()->get_filesystem());\n")
+			->write("\$asset = new \phpbb\\template\\asset(\$asset_file, \$this->env->get_path_helper(), \$this->env->get_filesystem());\n")
 			->write("if (substr(\$asset_file, 0, 2) !== './' && \$asset->is_relative()) {\n")
 			->indent()
 				->write("\$asset_path = \$asset->get_path();")
-				->write("\$local_file = \$this->getEnvironment()->get_phpbb_root_path() . \$asset_path;\n")
+				->write("\$local_file = \$this->env->get_phpbb_root_path() . \$asset_path;\n")
 				->write("if (!file_exists(\$local_file)) {\n")
 				->indent()
-					->write("\$local_file = \$this->getEnvironment()->findTemplate(\$asset_path);\n")
+					->write("\$local_file = \$this->env->findTemplate(\$asset_path);\n")
 					->write("\$asset->set_path(\$local_file, true);\n")
 				->outdent()
 				->write("}\n")
-				->write("\$asset->add_assets_version('{$config['assets_version']}');\n")
 			->outdent()
 			->write("}\n")
-			->write("\$this->getEnvironment()->get_assets_bag()->add_{$this->get_setters_name()}(\$asset);")
+			->write("\n")
+			->write("if (\$asset->is_relative()) {\n")
+			->indent()
+			->write("\$asset->add_assets_version(\$this->env->get_phpbb_config()['assets_version']);\n")
+			->outdent()
+			->write("}\n")
+			->write("\$this->env->get_assets_bag()->add_{$this->get_setters_name()}(\$asset);")
 		;
 	}
 
